"""YouTube DownLoad."""

#  playlists
# …
# entries

#  playlists / entries
# title
# id

#  playlist
# entries

#  playlist / entries
# …

#  videos
# id
# channel
# channel_id
# title
# channel_follower_count
# description
# tags
# thumbnails
# uploader_id
# uploader
# entries

#  videos / entries
# id
# title
# description truncated
# duration
# thumbnails
# view_count

from abc import ABC, abstractmethod
from enum import Enum
from rwx import Object
from rwx.log import stream as log
from yt_dlp import YoutubeDL


class Action(Enum):
    DOWNLOAD_VIDEO = 0
    EXTRACT_PLAYLIST = 1
    EXTRACT_PLAYLISTS = 2
    EXTRACT_VIDEO = 3
    EXTRACT_VIDEOS = 4


class Tab(Object, ABC):
    """YouTube Tab."""

    URL_ROOT = "https://youtube.com"

    def __init__(self, object_id: str) -> None:
        """Set object id.

        :param object_id: object identifier
        :type object_id: str
        """
        self.object_id = object_id
        log.info(self.object_id)
        self.url = self.get_url()
        log.info(self.url)
        self.info = Tab.yt_dl(self.get_options()).extract_info(self.url, download=False)

    def get_options(self) -> dict:
        """Return options for the action.

        :rtype: dict
        """
        return {
            "extract_flat": True,
            "skip_download": True,
        }

    @abstractmethod
    def get_url(self) -> str:
        """Return URL to access for object.

        :rtype: str
        """

    @staticmethod
    def yt_dl(opt: dict) -> YoutubeDL:
        options = {**opt, "ignoreerrors": False, "quiet": False}
        log.info(options)
        return YoutubeDL(options)


class Playlist(Tab):
    def __init__(self, playlist_id: str) -> None:
        self.playlist_id = playlist_id

    def get_url(self) -> str:
        return f"{Tab.URL_ROOT}/playlist?list={self.playlist_id}"


class Playlists(Tab):
    def __init__(self, channel_id: str) -> None:
        self.channel_id = channel_id

    def get_url(self) -> str:
        return f"{Tab.URL_ROOT}/channel/{self.channel_id}/playlists"


# id
# title
# formats
# thumbnails
# thumbnail
# description
#
# duration
# view_count
# categories
# tags
# fulltitle
class Video(Tab):
    def __init__(self, video_id: str) -> None:
        self.video_id = video_id

    def download(self) -> None:
        Tab.yt_dl(
            {
                "format": "bestvideo[ext=mp4]+bestaudio[ext=mp4]",
                "outtmpl": "%(id)s.%(ext)s",
                "writesubtitles": True,
                "writethumbnail": True,
            }
        ).download([self.url])

    def get_url(self) -> str:
        return f"{Tab.URL_ROOT}/watch?v={self.video_id}"


class Videos(Tab):
    def __init__(self, channel_id: str) -> None:
        self.channel_id = channel_id

    def get_url(self) -> str:
        return f"{Tab.URL_ROOT}/channel/{self.channel_id}/videos"


def command(action: Action):
    log.info(options)
    with dl(options) as youtube:
        match action:
            case _:
                youtube.extract_info(url, download=False)
